' Interrupt driven software serial receive UART Demo
' The routines held within "Buffered Rsin.Inc" replace the compiler's Rsout and Rsin command (with and without timeouts)
'
    $define Rsin_BufferSize 30              ' The size of the Rsin Buffer (max 255) (must be issued before the "Buffered RSIn" include)
'
' Configure the Alphanumeric LCD for use with the LCD Shield
'
    Declare LCD_DTPin = PORTB.4             ' LCD's data port starts at bit-4 of PortB
    Declare LCD_ENPin = PORTB.3             ' LCD's EN pin connection
    Declare LCD_RSPin = PORTB.2             ' LCD's RS pin connection
    Declare LCD_Interface = 4               ' LCD operating in 4-bit mode
    Declare LCD_Lines = 2                   ' LCD has 2 lines
    Declare LCD_Type = Alphanumeric         ' LCD type is alphanumeric (Hitachi)
    Declare LCD_CommandUs = 2000            ' Delay between command values sent to the LCD (in microseconds)
    Declare LCD_DataUs = 50                 ' Delay between data values sent to the LCD (in microseconds)
'
' Create some variables for the demonstration
'
    Dim ByteVar As Byte                     ' Byte to be received and displayed
    Dim bLCD_Xpos As Byte                   ' LCD's X position
    Dim bLCD_Ypos As Byte                   ' LCD's Y position

    Include "Buffered Rsin.inc"             ' Load the software UART routines into the program
                                            ' Preferably after the user variables have been created
'---------------------------------------------------------------------------------
    GoTo Main                               ' Jump over the subroutines
'---------------------------------------------------------------------------------
' Display a character on the LCD with boundary checks
'
' Input     : ByteVar holds the character to display
' Output    : None
' Notes     : Moves to alternate lines when the amount of character per line is exceeded
'
DisplayChar:
    If bLCD_Xpos > 16 Then                   ' Have we reached the end of the line?
        bLCD_Xpos = 1                        ' Yes. So reset the X position
        If bLCD_Ypos = 1 Then                ' Are we on line 1 of the LCD?
            Inc bLCD_Ypos                    ' Yes. So move to line 2
        Else                                 ' Otherwise...
            Dec bLCD_Ypos                    ' Move to line 1
        EndIf
        Cursor bLCD_Ypos, bLCD_Xpos          ' Move the cursor
    EndIf
    Print ByteVar                            ' Display the character
    Inc bLCD_Xpos                            ' Increment the X position
    Return
'---------------------------------------------------------------------------------
' Main demo loop
Main:
    DelayMS 100                             ' Wait for the LCD to stabilise
    Cls                                     ' Clear the LCD

    RSOut "Type some characters in the terminal window\r",_
           "and they will appear on the LCD.\r",_
           "Note that no characters are missed\r",_
           "even though there is a stupidly large delay within the receiving loop.\r\r"

    SetTimeout(5000)                        ' Set the timeout to 5 seconds

    bLCD_Xpos = 1                           ' Reset the LCD cursor's X position
    bLCD_Ypos = 1                           ' Reset the LCD cursor's Y position
    While 1 = 1                             ' Create an infinite loop
        While 1 = 1                         ' Create another infinite loop
            'ByteVar = RSIn                 ' Receive a byte from the serial UART (without timeout)
            ByteVar = RSIn,{Timeout}        ' Receive a byte from the serial UART (with timeout)
            DelayMS 200               ' << Add a stupidly large delay between characters received
            GoSub DisplayChar               ' Display the byte on the LCD
        Wend                                ' Close the loop
Timeout:                                    ' Come here is a timeout occured
        Cls                                 ' Clear the LCD
        Print At 2,1,"Timed Out"            ' Display the timeout message on line 2 of the LCD
        DelayMS 400                         ' Clear the LCD
        Cls
        bLCD_Xpos = 1                       ' Reset the LCD cursor's X position
        bLCD_Ypos = 1                       ' Reset the LCD cursor's Y position
    Wend                                    ' Do it forever


